// Thermistor lookup table for one thermistor or
// identical thermistors in all places.

/*
   This table doesn't depend on the type of electronics, but on the type of
   thermistor(s) you use. You want one table for each thermistor type you use.
*/

// How many thermistor tables we have.
#define NUMTABLES 2

// Names for our tables, so you can use them in config.h.
// Table numbering starts at 0.
#define THERMISTOR_EXTRUDER   0
#define THERMISTOR_BED   1

/*
   You may be able to improve the accuracy of this table in various ways.

   1. Measure the actual resistance of the resistor. It's "nominally" 4.7K,
      but that's ± 5%.
   2. Measure the actual beta of your thermistor:
      http://reprap.org/wiki/MeasuringThermistorBeta
   3. Generate more table entries than you need, then trim down the ones
      in uninteresting ranges.

   In either case you'll have to regenerate this table with
   createTemperatureLookup.py, which requires python, which is difficult to
   install on windows. Since you'll have to do some testing to determine the
   correct temperature for your application anyway, you may decide that the
   effort isn't worth it. Who cares if it's reporting the "right" temperature
   as long as it's keeping the temperature steady enough to print, right?
*/

// The number of value pairs in our table.
// Must be the same for all tables.
#define NUMTEMPS 102

uint16_t const temptable[NUMTABLES][NUMTEMPS][2] PROGMEM = {

// Table for the Extruder.
// The thermistor used for this table was an Epocs B57560G104F

// Thermistor lookup table
// default thermistor lookup table
// You may be able to improve the accuracy of this table in various ways.
//   1. Measure the actual resistance of the resistor. It's "nominally" 4.7K, but that's T 5%.
//   2. Measure the actual beta of your thermistor:http://reprap.org/wiki/MeasuringThermistorBeta
//   3. Generate more table entries than you need, then trim down the ones in uninteresting ranges.
// In either case you'll have to regenerate this table, which requires python, which is difficult to install on windows.
// Since you'll have to do some testing to determine the correct temperature for your application anyway, you
// may decide that the effort isn't worth it. Who cares if it's reporting the "right" temperature as long as it's
// keeping the temperature steady enough to print, right?
// ./createTemperatureLookup.py --r0=100000 --t0=25 --r1=0 --r2=4700 --beta=4085 --max-adc=1023
// r0: 100000
// t0: 25
// r1: 0
// r2: 4700
// beta: 4085
// max adc: 1023
// {ADC, temp*4 }, // temp

{
   {1, 3308}, // 827.022321272 C
   {11, 1576}, // 394.249338047 C
   {21, 1318}, // 329.59552951 C
   {31, 1184}, // 296.044708713 C
   {41, 1095}, // 273.959517596 C
   {51, 1030}, // 257.714817809 C
   {61, 979}, // 244.963691548 C
   {71, 938}, // 234.516924348 C
   {81, 902}, // 225.694029376 C
   {91, 872}, // 218.07151888 C
   {101, 845}, // 211.369067766 C
   {111, 821}, // 205.392114247 C
   {121, 800}, // 200.0004521 C
   {131, 780}, // 195.089911466 C
   {141, 762}, // 190.58111034 C
   {151, 745}, // 186.412249226 C
   {161, 730}, // 182.534329079 C
   {171, 715}, // 178.90788031 C
   {181, 702}, // 175.500666454 C
   {191, 689}, // 172.286035132 C
   {201, 676}, // 169.241710051 C
   {211, 665}, // 166.348890334 C
   {221, 654}, // 163.591568319 C
   {231, 643}, // 160.956005486 C
   {241, 633}, // 158.430324676 C
   {251, 624}, // 156.004189091 C
   {261, 614}, // 153.668546918 C
   {271, 605}, // 151.415426171 C
   {281, 596}, // 149.237768409 C
   {291, 588}, // 147.129292829 C
   {301, 580}, // 145.084384343 C
   {311, 572}, // 143.098000735 C
   {321, 564}, // 141.165595138 C
   {331, 557}, // 139.283050882 C
   {341, 549}, // 137.446626428 C
   {351, 542}, // 135.652908556 C
   {361, 535}, // 133.898772354 C
   {371, 528}, // 132.181346851 C
   {381, 521}, // 130.497985346 C
   {391, 515}, // 128.846239668 C
   {401, 508}, // 127.223837749 C
   {411, 502}, // 125.628663975 C
   {421, 496}, // 124.058741911 C
   {431, 490}, // 122.512219027 C
   {441, 483}, // 120.987353142 C
   {451, 477}, // 119.482500314 C
   {461, 471}, // 117.996103991 C
   {471, 466}, // 116.526685221 C
   {481, 460}, // 115.072833774 C
   {491, 454}, // 113.633200044 C
   {501, 448}, // 112.206487604 C
   {511, 443}, // 110.791446334 C
   {521, 437}, // 109.386865996 C
   {531, 431}, // 107.991570214 C
   {541, 426}, // 106.604410753 C
   {551, 420}, // 105.224262044 C
   {561, 415}, // 103.850015891 C
   {571, 409}, // 102.480576293 C
   {581, 404}, // 101.114854318 C
   {591, 399}, // 99.751762985 C
   {601, 393}, // 98.3902120692 C
   {611, 388}, // 97.0291027897 C
   {621, 382}, // 95.6673222952 C
   {631, 377}, // 94.3037378818 C
   {641, 371}, // 92.9371908593 C
   {651, 366}, // 91.5664899756 C
   {661, 360}, // 90.1904042921 C
   {671, 355}, // 88.8076553923 C
   {681, 349}, // 87.4169087802 C
   {691, 344}, // 86.0167643032 C
   {701, 338}, // 84.6057454 C
   {711, 332}, // 83.1822869347 C
   {721, 326}, // 81.744721327 C
   {731, 321}, // 80.2912626238 C
   {741, 315}, // 78.8199880757 C
   {751, 309}, // 77.3288166743 C
   {761, 303}, // 75.8154839697 C
   {771, 297}, // 74.2775123093 C
   {781, 290}, // 72.7121754002 C
   {791, 284}, // 71.1164557834 C
   {801, 277}, // 69.4869933872 C
   {811, 271}, // 67.8200227453 C
   {821, 264}, // 66.1112956776 C
   {831, 257}, // 64.3559851157 C
   {841, 250}, // 62.5485641857 C
   {851, 242}, // 60.6826523893 C
   {861, 235}, // 58.750817393 C
   {871, 226}, // 56.7443159474 C
   {881, 218}, // 54.6527498161 C
   {891, 209}, // 52.4636005979 C
   {901, 200}, // 50.1615879627 C
   {911, 190}, // 47.7277635319 C
   {921, 180}, // 45.13819683 C
   {931, 169}, // 42.3620090733 C
   {941, 157}, // 39.3583197645 C
   {951, 144}, // 36.0712873329 C
   {961, 129}, // 32.4215955693 C
   {971, 113}, // 28.29077593 C
   {981, 93}, // 23.4895466226 C
   {991, 70}, // 17.685172886 C
   {1001, 40}, // 10.1997836087 C
   {1010, 2} // 0.60324141619 C
},



// Thermistor table for the Heatbed.
// The thermistor used for this table was an 22K
// ./createTemperatureLookup.py --r0=22000 --t0=25 --r1=0 --r2=1500 --beta=3436 --max-adc=1023 //--num-temps=100
// {ADC, temp*4 }, // temp

{
{1, 6109}, // 1527.3787114 C
{11, 2091}, // 522.948857425 C
{21, 1671}, // 417.837332793 C
{31, 1465}, // 366.45332207 C
{41, 1334}, // 333.725631999 C
{51, 1240}, // 310.185416942 C
{61, 1168}, // 292.014710177 C
{71, 1109}, // 277.32446321 C
{81, 1060}, // 265.053184854 C
{91, 1018}, // 254.549831093 C
{101, 981}, // 245.388630849 C
{111, 949}, // 237.277104341 C
{121, 920}, // 230.006360438 C
{131, 893}, // 223.422417083 C
{141, 869}, // 217.408756003 C
{151, 847}, // 211.875240572 C
{161, 827}, // 206.750815404 C
{171, 807}, // 201.978546287 C
{181, 790}, // 197.512159352 C
{191, 773}, // 193.313569738 C
{201, 757}, // 189.351080584 C
{211, 742}, // 185.598046649 C
{221, 728}, // 182.031866629 C
{231, 714}, // 178.633212277 C
{241, 701}, // 175.385430935 C
{251, 689}, // 172.274076958 C
{261, 677}, // 169.286540242 C
{271, 665}, // 166.411748793 C
{281, 654}, // 163.639928416 C
{291, 643}, // 160.962406903 C
{301, 633}, // 158.371453245 C
{311, 623}, // 155.860144629 C
{321, 613}, // 153.422255682 C
{331, 604}, // 151.052165639 C
{341, 594}, // 148.744780088 C
{351, 585}, // 146.495464604 C
{361, 577}, // 144.299988179 C
{371, 568}, // 142.154474756 C
{381, 560}, // 140.055361482 C
{391, 551}, // 137.999362601 C
{401, 543}, // 135.983438058 C
{411, 536}, // 134.004766095 C
{421, 528}, // 132.060719207 C
{431, 520}, // 130.148842962 C
{441, 513}, // 128.266837251 C
{451, 505}, // 126.412539623 C
{461, 498}, // 124.583910389 C
{471, 491}, // 122.779019254 C
{481, 483}, // 120.996033245 C
{491, 476}, // 119.233205761 C
{501, 469}, // 117.488866569 C
{511, 463}, // 115.761412616 C
{521, 456}, // 114.049299521 C
{531, 449}, // 112.351033648 C
{541, 442}, // 110.665164647 C
{551, 435}, // 108.990278382 C
{561, 429}, // 107.324990162 C
{571, 422}, // 105.667938179 C
{581, 416}, // 104.017777104 C
{591, 409}, // 102.373171741 C
{601, 402}, // 100.732790678 C
{611, 396}, // 99.0952998509 C
{621, 389}, // 97.4593559472 C
{631, 383}, // 95.8235995557 C
{641, 376}, // 94.1866479714 C
{651, 370}, // 92.5470875513 C
{661, 363}, // 90.9034655019 C
{671, 357}, // 89.2542809632 C
{681, 350}, // 87.5979752312 C
{691, 343}, // 85.9329209353 C
{701, 337}, // 84.2574099485 C
{711, 330}, // 82.5696397696 C
{721, 323}, // 80.8676980562 C
{731, 316}, // 79.1495449216 C
{741, 309}, // 77.4129925158 C
{751, 302}, // 75.6556812959 C
{761, 295}, // 73.8750522447 C
{771, 288}, // 72.068314097 C
{781, 280}, // 70.23240438 C
{791, 273}, // 68.3639427289 C
{801, 265}, // 66.4591744832 C
{811, 258}, // 64.5139019408 C
{821, 250}, // 62.5233997871 C
{831, 241}, // 60.4823100156 C
{841, 233}, // 58.3845099517 C
{851, 224}, // 56.2229445363 C
{861, 215}, // 53.9894104279 C
{871, 206}, // 51.6742740925 C
{881, 197}, // 49.2660978075 C
{891, 187}, // 46.7511345786 C
{901, 176}, // 44.1126321106 C
{911, 165}, // 41.3298512504 C
{921, 153}, // 38.3766443514 C
{931, 140}, // 35.2193309862 C
{941, 127}, // 31.8134039379 C
{951, 112}, // 28.098187698 C
{961, 95}, // 23.9876852975 C
{971, 77}, // 19.3537566538 C
{981, 55}, // 13.9922257087 C
{991, 30}, // 7.545331416 C
{1000, 0} // 0.23133287307 C
 }

};
